/*
 * KCall VoIP/CTI telephony
 * Copyright (C) 2006-2007 by basyskom GmbH
 *  @author Mike Hauth <info@basyskom.de>
 *  @author Tobias Hunger <info@basyskom.de>
 *  @author Eva Brucherseifer <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _CALLMODEL_H_
#define _CALLMODEL_H_

#include <QObject>
#include <QList>
#include <QAbstractTableModel>

#include "callinfo.h"

class KCallCore;
class CallModelPrivate;

namespace QtTapioca
{
class Contact;
class Connection;
class StreamChannel;
}

/**
 *  List of Past Calls
 */

class CallModel : public QAbstractTableModel
{
        Q_OBJECT
    public:
        explicit CallModel(KCallCore * const core, QObject * parent = 0);
        ~CallModel();

        // QtAbstractTableModel implementation:
        int rowCount(const QModelIndex & parent = QModelIndex()) const;
        int columnCount(const QModelIndex & parent = QModelIndex()) const;
        QVariant data(const QModelIndex & index, int role = Qt::DisplayRole) const;
        QVariant headerData (int section, Qt::Orientation orientation,
                             int role = Qt::DisplayRole ) const;

        // Custom methods:
        /**
         * @brief Set the call's subject.
         * @param row The row number to change.
         * @param subject The subject to set.
         */
        void setSubject(const int row, const QString & subject);
        bool hasDTMFSupport(const QString & id);

        enum ColumnNames
        {
            CALL_ID = 0,
            CALLER_NR,
            CALLER_NAME,
            SUBJECT,
            STATUS,
            DATE,
            TIME,
            COLUMN_MAX
        };

        // Slots (for D-Bus):
    public slots:
        void addChannel(QtTapioca::Connection * const,
                        QtTapioca::StreamChannel * const,
                        const bool incoming);

        void acceptCall(const QString & id);
        void hangupCall(const QString & id);
        void sendDTMF(const QString & id, const QChar tone);

    private slots:
        void onChannelClosed();
        void onContactChanged(QtTapioca::Contact *);

    private:
        CallModelPrivate * const d;
};

#endif
