/*
 * Copyright (C) 2007 by basyskom GmbH
 *  @author Tobias Hunger <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "dtmfinput.h"

#include <kdebug.h>
#include <QDialog>

#include "ui_dtmfinput.h"
#include "kcallcore.h"
#include "callmodel.h"

class DTMFInputPrivate
{
public:
    DTMFInputPrivate(KCallCore * c) :
        core(c)
    { }
    ~DTMFInputPrivate() { }

    void setGuiEnabled(const bool enabled)
    {
        ui.button0->setEnabled(enabled);
        ui.button1->setEnabled(enabled);
        ui.button2->setEnabled(enabled);
        ui.button3->setEnabled(enabled);
        ui.button4->setEnabled(enabled);
        ui.button5->setEnabled(enabled);
        ui.button6->setEnabled(enabled);
        ui.button7->setEnabled(enabled);
        ui.button8->setEnabled(enabled);
        ui.button9->setEnabled(enabled);
        ui.button0->setEnabled(enabled);
        ui.buttonAsterisk->setEnabled(enabled);
        ui.buttonHash->setEnabled(enabled);
    }

    KCallCore * const core;

    Ui::DTMFInput ui;

    QString currentId;
};

DTMFInput::DTMFInput(KCallCore * const core, QWidget *parent) :
       QWidget(parent),
       d(new DTMFInputPrivate(core))
{
    d->ui.setupUi(this);
    d->setGuiEnabled(false);
    connect(d->ui.button0, SIGNAL(clicked()), this, SLOT(sendNumber()));
    connect(d->ui.button1, SIGNAL(clicked()), this, SLOT(sendNumber()));
    connect(d->ui.button2, SIGNAL(clicked()), this, SLOT(sendNumber()));
    connect(d->ui.button3, SIGNAL(clicked()), this, SLOT(sendNumber()));
    connect(d->ui.button4, SIGNAL(clicked()), this, SLOT(sendNumber()));
    connect(d->ui.button5, SIGNAL(clicked()), this, SLOT(sendNumber()));
    connect(d->ui.button6, SIGNAL(clicked()), this, SLOT(sendNumber()));
    connect(d->ui.button7, SIGNAL(clicked()), this, SLOT(sendNumber()));
    connect(d->ui.button8, SIGNAL(clicked()), this, SLOT(sendNumber()));
    connect(d->ui.button9, SIGNAL(clicked()), this, SLOT(sendNumber()));
    connect(d->ui.buttonAsterisk, SIGNAL(clicked()), this, SLOT(sendAsterisk()));
    connect(d->ui.buttonHash, SIGNAL(clicked()), this, SLOT(sendHash()));
}

DTMFInput::~DTMFInput()
{ delete d; }

void DTMFInput::switchChannel(const QString & id)
{
    d->currentId = id;
    d->setGuiEnabled(d->core->callModel()->hasDTMFSupport(id));
}

void DTMFInput::sendNumber()
{
    QObject * button = sender();
    if (0 == button) { return; }

    QString button_name = button->objectName();
    if (!button_name.startsWith("button") ||
        button_name.size() != 7) { return; }

    emit sendDTMF(d->currentId, button_name[6]);
}

void DTMFInput::sendAsterisk()
{ emit sendDTMF(d->currentId, QChar('*')); }

void DTMFInput::sendHash()
{ emit sendDTMF(d->currentId, QChar('#')); }
