/*
 * Copyright (C) 2007 by basyskom GmbH
 *  @author Eva Brucherseifer <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <QLayout>
#include <QVBoxLayout>


#include <kcomponentdata.h>
#include <klocale.h>
#include <kparts/genericfactory.h>
#include <kparts/statusbarextension.h>
#include <kaction.h>
#include <kdebug.h>
#include <kstatusbar.h>
#include <kiconloader.h>

#include <QtDBus/QtDBus>
#include "kcallpartadaptor.h"
#include "kcallcore.h"

#include "kcall_part.h"

typedef KParts::GenericFactory< KCallPart > KCallFactory;
K_EXPORT_COMPONENT_FACTORY( libkcallpart, KCallFactory )

KCallPart::KCallPart( QWidget *parentWidget, QObject *parent,
                                            const QStringList & )
    : KParts::ReadOnlyPart( parent )
{
    setComponentData( KCallFactory::componentData() );

  // create a canvas to insert our widget
    QWidget *canvas = new QWidget( parentWidget );
    canvas->setFocusPolicy( Qt::ClickFocus );
    setWidget( canvas );

    QVBoxLayout *topLayout = new QVBoxLayout( canvas );

    KIconLoader::global()->addAppDir( "kcall" );

    mCore = new KCallCore( this, true, canvas );
    mCore->restoreSettings();
    topLayout->addWidget( mCore->widget() );

    KParts::StatusBarExtension *statusBar = new KParts::StatusBarExtension( this );
    mCore->setStatusBar( statusBar->statusBar() );

    setXMLFile( "kcall_part.rc" );

    new CoreAdaptor( this );
    QDBusConnection::sessionBus().registerObject( "/KCall", this, QDBusConnection::ExportAdaptors );
}

KCallPart::~KCallPart()
{
//    mCore->save();
    mCore->saveSettings();

//    KABPrefs::instance()->writeConfig();
    closeUrl();
}

KAboutData *KCallPart::createAboutData()
{
    return KCallCore::createAboutData();
}

void KCallPart::save()
{
//    mCore->save();
}

bool KCallPart::newCall( const QString &number )
{
    return mCore->newCall(number);
}

void KCallPart::exit()
{
    mCore->queryClose();

    delete this;
}

bool KCallPart::openURL( const KUrl &url )
{
    kDebug(5720) << "KCallPart:openFile()" << endl;

    mCore->widget()->show();

//    if ( !url.isEmpty() )
//        mCore->importVCard( url );

    emit setWindowCaption( url.prettyUrl() );

    return true;
}

bool KCallPart::openFile()
{
    return false;
}

bool KCallPart::handleCommandLine()
{
    return mCore->handleCommandLine();
}

void KCallPart::guiActivateEvent( KParts::GUIActivateEvent *e )
{
    kDebug(5720) << "KCallPart::guiActivateEvent" << endl;
    KParts::ReadOnlyPart::guiActivateEvent( e );

/*    if ( e->activated() )
      mCore->reinitXMLGUI();

    if ( !e->activated() ) {
        mCore->statusBar()->removeItem( 1 );
        mCore->statusBar()->removeItem( 2 );
    }*/
}

#include "kcall_part.moc"
