/*
 * Copyright (C) 2006-2007 by basyskom GmbH
 *  @author Mike Hauth <info@basyskom.de>
 *  @author Eva Brucherseifer <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "kcallview.h"

#include <kdebug.h>
#include <kplugininfo.h>
#include <kapplication.h>

#include <QRegExp>
#include <QList>
#include <QToolButton>
#include <QVBoxLayout>
#include <QHeaderView>
#include <QTableView>

#include "vcardview.h"
#include "kcallsettings.h"
#include "kcallcore.h"
#include "historymodel.h"
#include "callmodel.h"

KCallView::KCallView(KCallCore * const core, QWidget *parent) :
       QWidget(parent),
       m_core(core)
{
    ui.setupUi(this);

    //load the history and completion list for ui.nrInput
    QStringList nrInputHistoryList;
    nrInputHistoryList = KCallSettings::completion();
    kDebug() << nrInputHistoryList << endl;
    ui.nrInput->completionObject()->setItems( nrInputHistoryList );
    //nrInput->setHistoryItems( nrInputHistoryList, true ); //won´t work, so the dirty version

    for ( QStringList::Iterator it = nrInputHistoryList.begin(); it != nrInputHistoryList.end(); ++it ) {
        ui.nrInput->addToHistory(*it);
    }

    // init dialing
    ui.acceptButton->setIcon( KIcon("phone_accept") );
    ui.hangupButton->setIcon( KIcon("phone_reject") );
    //hangupButton->setIconSet( SmallIconSet("phone_reject",22 ) );

    //init activeCallListView-List
//     ui.activeCallListView->addColumn("Caller",200);
//     ui.activeCallListView->addColumn("Subject",0);
//     m_callStatusCol = ui.activeCallListView->addColumn("Status",60);
//     m_callIdCol = ui.activeCallListView->addColumn("CallId",0);
//     m_actCallNrCol = ui.activeCallListView->addColumn("CallNr",0);

    //init phonebookList
    ui.phonebookList->addColumn("Name");
    m_pbLCallNrCol = ui.phonebookList->addColumn("PhoneNumber");
    ui.phonebookList->addColumn("ABId",0);

    updatePhonebookView();

    //init historyTableView
/** @todo port to decibel history api and/or save length in local settings*/
/*
    KCallSipSettings::self()->config()->setGroup("CallLists");
*/
    ui.historyTableView->setModel(m_core->historyModel());
    ui.historyTableView->verticalHeader()->hide();
    ui.historyTableView->hideColumn(HistoryModel::CALL_ID);
    ui.historyTableView->hideColumn(HistoryModel::CALLER_NAME);
    ui.historyTableView->hideColumn(HistoryModel::SUBJECT);
    ui.historyTableView->hideColumn(HistoryModel::CALLER_NAME);
    ui.historyTableView->hideColumn(HistoryModel::TIME);
    ui.historyTableView->setSelectionMode(QAbstractItemView::NoSelection);

    ui.activeCallTableView->setModel(m_core->callModel());
    ui.activeCallTableView->verticalHeader()->hide();
    ui.activeCallTableView->hideColumn(HistoryModel::CALL_ID);
    ui.activeCallTableView->hideColumn(HistoryModel::CALLER_NAME);
    ui.activeCallTableView->hideColumn(HistoryModel::SUBJECT);
    ui.activeCallTableView->hideColumn(HistoryModel::CALLER_NAME);
    ui.activeCallTableView->hideColumn(HistoryModel::TIME);
    ui.activeCallTableView->setSelectionMode(QAbstractItemView::SingleSelection);
    ui.activeCallTableView->setSelectionBehavior(QAbstractItemView::SelectRows);

    QVBoxLayout* vCardLayout = new QVBoxLayout( ui.info );
    vCardLayout->setObjectName( "vCardLayout" );
//    , 11, 6, "vCardLayout");
    m_vcardView = new VCardView(m_core->historyModel(), ui.info);
    vCardLayout->addWidget(m_vcardView);
    connect(m_vcardView, SIGNAL(dialNr(const QString&)), this, SLOT(dialByInput(const QString&)));

    ui.phonebookList->setSelected(ui.phonebookList->firstChild(), true);

    setDialTab();

    connect( ui.acceptButton, SIGNAL(clicked()), this, SLOT(acceptCall()));
//    connect( ui.activeCallListView, SIGNAL(clicked(Q3ListViewItem*)), this, SLOT(setVCard(Q3ListViewItem*)));
    connect( ui.nrInput, SIGNAL(returnPressed(const QString&)), this, SLOT(dialByInput(const QString&)));
    connect( ui.nrInput, SIGNAL(textChanged(const QString&)), ui.nrInput, SLOT(setCompletedText(const QString&)));
    connect( ui.phonebookList, SIGNAL(clicked(Q3ListViewItem*)), this, SLOT(setVCard(Q3ListViewItem*)));
    connect( m_vcardView, SIGNAL( phoneNumberClicked(QString& )), this, SLOT( dialByInput(const QString&)));
    connect( ui.acceptButton, SIGNAL(clicked()), this, SLOT(decideAcceptAction()));

    connect(ui.activeCallTableView->selectionModel(),
            SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection&)),
            this, SLOT(setButtonActions()));

    setButtonActions();
}

KCallView::~KCallView()
{
    kDebug() << "KCallView::destroy()" << endl;

    KCallSettings::setCompletion(ui.nrInput->completionObject()->items());
    KCallSettings::setHistory(ui.nrInput->historyItems());
    KCallSettings::self()->writeConfig();
}

void KCallView::newCall(QString callid, QString caller, QString subject, CallStatus status)
{
    kDebug() << "KCallView::newCall()" << endl;
    kDebug() << callid << endl;
    CallDesc tmpDesc;
    tmpDesc.subject = subject;
    tmpDesc.callerUser = caller;
    //tmpDesc.callerProxy = "";
    tmpDesc.status = status;

    m_callsInList.insert(callid, tmpDesc);

    QString callerText = caller;
}

void KCallView::dialByInput(const QString &phoneNr)
{
    kDebug() << "KCallView::dialByInput()" << endl;

    ui.nrInput->insertURL(phoneNr);
    kDebug() << "PhoneNumber" << phoneNr << endl;
    emit dial(phoneNr);
}

void KCallView::decideAcceptAction()
{
    kDebug() << "KCallView::decideAcceptAction()" << endl;

    QItemSelectionModel * selection = ui.activeCallTableView->selectionModel();
    if (selection->selectedRows().isEmpty())
    {
        //kDebug() << "Nummer aus Input:" << ui.nrInput->currentText()<<endl;
        dialByInput(ui.nrInput->currentText());
    }
}

void KCallView::acceptCall()
{
    kDebug() << "KCallView::acceptCall()" << endl;

    QItemSelectionModel * selection = ui.activeCallTableView->selectionModel();

    QModelIndexList rows = selection->selectedRows();
    QModelIndex current_row;
    foreach  (current_row, rows)
    {
        QString call_id = m_core->callModel()->data(current_row).toString();
        emit accept(call_id);
    }

    selection->clear();
}

void KCallView::rejectCall()
{
    kDebug() << "KCallView::rejectCall()" << endl;

    QItemSelectionModel * selection = ui.activeCallTableView->selectionModel();

    QModelIndexList rows = selection->selectedRows();
    QModelIndex current_row;
    foreach  (current_row, rows)
    {
        QString call_id = m_core->callModel()->data(current_row).toString();
        emit reject(call_id);
    }

    selection->clear();
}

void KCallView::hangupCall()
{
    kDebug() << "KCallView::hangupCall()" << endl;

    QItemSelectionModel * selection = ui.activeCallTableView->selectionModel();

    QModelIndexList rows = selection->selectedRows();
    QModelIndex current_row;
    foreach  (current_row, rows)
    {
        QString call_id = m_core->callModel()->data(current_row).toString();
        emit hangup(call_id);
    }

    selection->clear();
}

void KCallView::cancelCall(QString callId)
{
    kDebug() << "KCallView::cancelCall()" << endl;

//     Q3ListViewItem *call;
//     call = ui.activeCallListView->findItem(callId, m_callIdCol);
//
//     setCallStatus(callId, CANCELED);
//     setButtonsActCalls();
//
//     if(call != 0)
//         ui.activeCallListView->takeItem(call);
}

#if 0
void KCallView::kCallStatus(bool online)
{
    kDebug() << "KCallView::kCallStatus()" << endl;

    ui.nrInput->setEnabled(online);
    ui.acceptButton->setEnabled(online);
    ui.hangupButton->setEnabled(online);
}
#endif

void KCallView::updatePhonebookView()
{
    kDebug() << "KCallView::updatePhonebookView()" << endl;

    m_core->addressBook()->load();
    KABC::AddressBook::ConstIterator it;

    ui.phonebookList->clear();
    for ( it = m_core->addressBook()->begin();
          it != m_core->addressBook()->end();
          ++it )
    {
        kDebug() << "Addressbook Entry: " << it->formattedName() << " - " << it->phoneNumbers().size() << endl;

        if(it->phoneNumbers().size() > 0)
        {
            for (int i=0; i<it->phoneNumbers().size() ; ++i)
              kDebug() << "Phone Number: " <<  it->phoneNumbers()[i].number() << endl;
            new Q3ListViewItem(ui.phonebookList, it->formattedName(), it->phoneNumbers()[0].number(),it->uid());
        }
        //for ( int i = 0; i < sumOfNrs; i++)  // only show one entry per person and main phone number
        {
            //new K3ListViewItem(phonebookList, it->formattedName(), it->phoneNumbers()[i].number(), it->uid());
        }
    }
}

void KCallView::setVCard(Q3ListViewItem* item)
{
    kDebug() << "KCallView::setVCard()" << endl;

    QString phoneNr = "";
    if(item != NULL )
    {
#if 0
        if(item == ui.activeCallListView->selectedItem())
        {
            phoneNr = item->text(actCallNrCol);
            ui.phonebookList->clearSelection();
            ui.historyTableView->clearSelection();
        }else
        if(item == ui.historyTableView->selectedItem())
        {
            phoneNr = item->text(hLCallNrCol);
            ui.activeCallListView->clearSelection();
            ui.phonebookList->clearSelection();
        }else
        if(item == ui.phonebookList->selectedItem())
        {
            phoneNr = item->text(pbLCallNrCol);
            ui.activeCallListView->clearSelection();
            ui.historyTableView->clearSelection();
        }
#endif
    }
    KABC::Addressee person = m_core->getPersonFromNumber(phoneNr);

    m_vcardView->setAddressee(person);
    m_vcardView->show();
}

void KCallView::setDialTab()
{
    kDebug() << "KCallView::setDialTab()" << endl;

//    if(nrTabs->currentPage() == phonebookTab)
    {
        ui.searchInput->setListView(ui.phonebookList);
        //setPhonebookList();
    }
    /*else
    if(callTabs->currentPage() == callListTab)
    {
        searchInput->setListView(historyListView);
        //setHistoryList();
    }
*/
}

void KCallView::setButtonActions()
{
    kDebug() << "KCallView::setButtonActions()" << endl;

    // no selection:
    ui.hangupButton->setText(i18n("Hang up"));
    ui.hangupButton->setEnabled(false);
    ui.acceptButton->setText(i18n("Dial"));
    ui.acceptButton->setEnabled(true);

    QItemSelectionModel * selection = ui.activeCallTableView->selectionModel();

    QModelIndexList rows = selection->selectedRows(CallModel::STATUS);
    QModelIndex current_index;
    foreach  (current_index, rows)
    {
        QVariant var(m_core->callModel()->data(current_index, Qt::UserRole));
        QVariant call_id(m_core->callModel()->data(
                         m_core->callModel()->index(current_index.row(),
                                           CallModel::CALL_ID)));
        kDebug() << "KCallView::setButtonActions(): var is" << var << endl;
        CallStatus status(CallStatus(var.toInt()));

        switch (status)
        {
        case INCOMING:
            kDebug() << "KCallView::setButtonActions(): INCOMING" << endl;
            disconnect(ui.hangupButton, SIGNAL(clicked()), this, SLOT(hangupCall()));
               connect(ui.hangupButton, SIGNAL(clicked()), this, SLOT(rejectCall()));
            ui.hangupButton->setText(i18n("Reject"));
            ui.hangupButton->setEnabled(true);
            ui.acceptButton->setText(i18n("Accept"));
            ui.acceptButton->setEnabled(true);
            break;
        case OUTGOING:
        case ACTIVE:
            kDebug() << "KCallView::setButtonActions(): OUTGOING/ACTIVE" << endl;
            disconnect(ui.hangupButton, SIGNAL(clicked()), this, SLOT(rejectCall()));
               connect(ui.hangupButton, SIGNAL(clicked()), this, SLOT(hangupCall()));
            ui.hangupButton->setText(i18n("Hang up"));
            ui.hangupButton->setEnabled(true);
            ui.acceptButton->setText(i18n("Dial"));
            ui.acceptButton->setEnabled(false);
            emit activeChannelSelected(call_id.toString());
            break;
        default:
            kDebug() << "KCallView::setButtonActions(): default (" << status << ")" << endl;
            break;
        }
    }
}
