/*
 * Copyright (C) 2006-2007 by basyskom GmbH
 *  @author Mike Hauth <info@basyskom.de>
 *  @author Eva Brucherseifer <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef KCALLVIEW_H
#define KCALLVIEW_H

#include <QString>
#include <QWidget>
#include <QListView>
#include <QPixmap>

#include "kcall_export.h"
#include "ui_kcallview.h"

#include "vcardview.h"
#include "callinfo.h"

class Q3ListViewItem;
class HistoryModel;
class CallModel;
class KCallCore;

struct KCALL_EXPORT CallDesc
{
    QString subject;
    QString callerUser;
    CallStatus status;
};

class KCALL_EXPORT KCallView : public QWidget
{
    Q_OBJECT

public:
    explicit KCallView(KCallCore * const, QWidget *parent = 0);
    ~KCallView();

public slots:
    void newCall(QString callid, QString caller, QString subject,
                 CallStatus status);
    void dialByInput(const QString &phoneNr);
    void decideAcceptAction();
    void acceptCall();
    void rejectCall();
    void hangupCall();
    void cancelCall(QString callId);
//    void setCallStatus(QString callid, CallStatus status);
    void updatePhonebookView();
//    void kCallStatus(bool online);
    void setVCard(Q3ListViewItem* item);
    void setDialTab();
    void setButtonActions();

signals:
    void accept(const QString &);
    void reject(const QString &);
    void hangup(const QString &);
    void dial(const QString &);

    void activeChannelSelected(const QString &);

protected:
    int m_callStatusCol;
    int m_actCallNrCol;
    int m_pbLCallNrCol;
    int m_hLCallNrCol;
    int m_callIdCol;

    VCardView *m_vcardView;
    HistoryModel * m_historyModel;
    CallModel * m_callModel;
    QMap <QString, CallDesc> m_callsInList;
    KCallCore * const m_core;

private:
    Ui::KCallView ui;
};

#endif
