/*
 * Copyright (C) 2006-2007 by basyskom GmbH
 *  @author Mike Hauth <info@basyskom.de>
 *  @author Eva Brucherseifer <info@basyskom.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "vcardview.h"

#include "historymodel.h"

#include <kdebug.h>
#include <kabc/addressee.h>
#include <kabc/address.h>
#include <kabc/picture.h>
#include <kurllabel.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kurllabel.h>

#include <QRegExp>
#include <QHeaderView>

VCardView::VCardView(HistoryModel * const base, QWidget *parent) :
    QWidget(parent)
 {
    ui.setupUi(this);

    labelLayout = new QVBoxLayout( ui.vcPhoneNumberLabels );
    nrLayout    = new QVBoxLayout( ui.vcPhoneNumbers );


    proxyModel.setSourceModel(base);
    // block everything, we have no addressee set up yet:
    proxyModel.setFilterRegExp(QRegExp("XXXX", Qt::CaseSensitive, QRegExp::FixedString));
    // set up Call history table:
    ui.vcCallHistoryTableView->setModel(&proxyModel);
    ui.vcCallHistoryTableView->verticalHeader()->hide();
    ui.vcCallHistoryTableView->hideColumn(HistoryModel::CALL_ID);
    ui.vcCallHistoryTableView->hideColumn(HistoryModel::CALLER_NR);
    ui.vcCallHistoryTableView->hideColumn(HistoryModel::CALLER_NAME);
    ui.vcCallHistoryTableView->hideColumn(HistoryModel::SUBJECT);
    ui.vcCallHistoryTableView->hideColumn(HistoryModel::CALLER_NAME);
    ui.vcCallHistoryTableView->hideColumn(HistoryModel::TIME);

    /// we don't need it currently
    ui.vcText->setReadOnly(true);
    ui.vcText->setPaletteBackgroundColor(palette().color(QPalette::Normal, QColorGroup::Background));
    ui.vcText->hide();

    ui.vcPicture->setFixedSize(80, 80);
    ui.vcPicture->setPixmap( KIconLoader::global()->loadIcon( "personal", KIconLoader::Desktop, 128 ) );
}

void VCardView::setAddressee(KABC::Addressee person)
{
    kDebug() << "VCardView::setAddressee()" << endl;

    ui.vcName->setText( person.formattedName() );

    //QString localTime = "Localtime:" + person.timeZone().toString();

    QString title = "";
    if( person.title() != "" )
        title = ", "+person.title()+"<br />";

    QString role = "";
    if(person.role() != "")
        role = person.roleLabel()+" "+person.role()+"<br />";

    QString organization = "";
    if(person.organization() != "")
        organization = person.organizationLabel()+" "+person.organization()+"<br />";

    QString prefPostAddress = "";
    KABC::Address address = person.address(KABC::Address::Pref);
    prefPostAddress = address.formattedAddress();

    setPhoneNumbers( person );

    QString vcInfoText = title+role+organization+prefPostAddress;
    ui.vcInfo->setText(vcInfoText);

    QPixmap photo = QPixmap(person.photo().data());
    if ( photo.size().isEmpty() == 0)
        photo = KIconLoader::global()->loadIcon( "personal", KIconLoader::Desktop, 128 );
    ui.vcPicture->setPixmap( photo );
    kDebug() << ui.vcPicture->size()<< endl;
}


void VCardView::setPhoneNumbers(KABC::Addressee person)
{
    kDebug() << "phoneNr::setPhoneNumbers()" << endl;

    // @todo: Update Filter RegExp on proxyModel to actually display data.

    QList<QLabel*>::iterator it;
    for ( it = phoneTypeLabels.begin(); it!=phoneTypeLabels.end() ; it++ ){
        kDebug() << "Loeschen:" << (*it)->text() << endl;
        delete *it;
    }
    phoneTypeLabels.clear();

    QList<KUrlLabel*>::iterator kit;
    for ( kit = phoneNumberLabels.begin(); kit!=phoneNumberLabels.end() ; kit++ ){
        kDebug() << "Loeschen:" << (*kit)->text() << endl;
        delete *kit;
    }

    phoneNumberLabels.clear();
    show();

    KABC::PhoneNumber::List nrList = person.phoneNumbers();
    KABC::PhoneNumber::List::Iterator phoneIter;

    for ( phoneIter = nrList.begin(); phoneIter != nrList.end(); ++phoneIter)
    {
        kDebug() << "Label:"<< (*phoneIter).typeLabel() << endl;
        kDebug() << "Nr:"<< (*phoneIter).number() << endl;

        QLabel *tmpLabel = new QLabel( ui.vcPhoneNumberLabels );
        labelLayout->addWidget(tmpLabel);
        tmpLabel->show();

        tmpLabel->setText((*phoneIter).typeLabel());
        tmpLabel->setAlignment( Qt::AlignRight );

        phoneTypeLabels.append(tmpLabel);

        KUrlLabel *tmpNr = new KUrlLabel( ui.vcPhoneNumbers );
        connect(tmpNr, SIGNAL(leftClickedUrl(const QString&)),this, SLOT(dialNumber(const QString&)));
        nrLayout->addWidget(tmpNr);
        tmpNr->show();

        tmpNr->setText((*phoneIter).number());
        tmpNr->setUrl((*phoneIter).number());
        tmpNr->setAlignment( Qt::AlignLeft );

        phoneNumberLabels.append(tmpNr);

    }
    ui.vcPhoneNumberLabels->show();
    ui.vcPhoneNumbers->show();
}


void VCardView::dialNumber(const QString &number)
{
    kDebug() << "VCardView::dialNumber(" << number << ")" << endl;
    emit phoneNumberClicked( number );
}
