/***************************************************************************
                          import.cpp  -  a dialog to import notes
                             -------------------
    begin                : Tue 25 Oct 2005
    copyright            : (C) 2005 by Michal Rudolf
    email                : mrudolf@kdewebdev.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "importdialog.h"

#include <kcombobox.h>
#include <kfiledialog.h>
#include <klineedit.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpushbutton.h>

#include <qfile.h>
#include <qtextstream.h>


ImportDialog::ImportDialog(QWidget* p) : KDialog(p)
{
  setupUi(this);
  connect(buttonOk, SIGNAL(clicked()), SLOT(accept()));
  connect(buttonCancel, SIGNAL(clicked()), SLOT(reject()));
  connect(buttonFiles, SIGNAL(clicked()), SLOT(selectFiles()));
}

ImportDialog::~ImportDialog()
{
}

int ImportDialog::exec()
{
  int result = KDialog::exec();
  m_titles.clear();
  m_notes.clear();
  if (result == QDialog::Accepted)
  {
    int flags = importOptions();
    for (QStringList::Iterator it = m_files.begin(); it != m_files.end(); ++it)
    {
      QString title = i18n("Untitled");
      QString note = importNote(*it, title, flags);
      if (note == QString::null)
        KMessageBox::error(0, i18n("<qt>Cannot open file<br><b>%1</b></qt>", *it));
      else
      {
        m_titles.append(title);
        m_notes.append(note);
      }
    }
  }
  return result;
}

void ImportDialog::selectFiles()
{
  m_files = KFileDialog::getOpenFileNames();
  if (m_files.count())
    editFiles->setText(m_files.join(" "));
}

QString ImportDialog::importNote(const QString& fileName, QString& title, int flags) const
{
  QFile file(fileName);
  if (!file.open(QIODevice::ReadOnly))
    return QString::null;
  QTextStream ts(&file);
  QString line;
  QString note = "<html><head><meta name=\"qrichtext\" content=\"1\" /></head><body>\n";
  if (flags & UseParagraphs)
    note.append("<p>");
#ifdef KDE4PORT_TEMPORARILY_REMOVED
  if (flags & UseUTF8)  // Set encoding
    ts.setEncoding(QTextStream::UnicodeUTF8);
#endif
  QString firstline;
  while (!( line = ts.readLine() ).isEmpty() )
  {
    if (firstline.isNull())
      firstline = line;
    if (line.trimmed().isEmpty() && flags & UseParagraphs)
    {
      if (!note.endsWith("<p>"))
        note.append("</p>\n<p>");
    }
    else
      note.append(line.replace("<", "&lt;"));
    if (flags & UseLines)
      note.append("<br />");
    else if (!note.endsWith(" "))
      note.append(" ");
  }
  if (flags & UseParagraphs)
    note.append("</p>\n");
  note.append("</body></html>\n");
  if (flags & UseFilename)
    title = fileName.section('/', -1, -1);
  else if (flags & UseFirstWord)
    title = firstline.trimmed().section(' ', 0, 0);
  else if (flags & UseFirstLine)
    title = firstline;
  else
    title = fileName;
  file.close();
  return note;
}

void ImportDialog::setImportOptions(int i)
{
  comboLines->setCurrentIndex(i % 4);
  comboEncoding->setCurrentIndex((i & UseUTF8) != 0);
  comboTitle->setCurrentIndex(((i & UseFilename) != 0) + ((i & UseFirstWord) != 0) * 2
     + ((i & UseFirstLine) != 0) * 3);
}

int ImportDialog::importOptions() const
{
  return comboLines->currentIndex() +
        (comboEncoding->currentIndex() == 1 ? UseUTF8 : UseLocal) +
        (comboTitle->currentIndex() == 1 ? UseFilename : 0) +
        (comboTitle->currentIndex() == 2 ? UseFirstWord : 0) +
        (comboTitle->currentIndex() == 3 ? UseFirstLine : 0);
}

int ImportDialog::noteCount() const
{
  return m_notes.count();
}

QString ImportDialog::noteText(int i) const
{
  return m_notes[i];
}

QString ImportDialog::noteTitle(int i) const
{
  return m_titles[i];
}
