/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_FEED_H
#define KRSS_FEED_H

#include "krss_export.h"
#include "item.h"
#include "tag.h"

#include <akonadi/collection.h>

#include <QtCore/QObject>

class KIcon;
class QString;

namespace KRss {

class ItemListJob;

class KRSS_EXPORT Feed : public QObject
{
    Q_OBJECT

public:

    typedef Akonadi::Collection::Id Id;

    enum ArchivingMode {
        Default,
        KeepAllArticles,
        LimitArticleNumber,
        LimitArticleAge
    };

public:

    explicit Feed( QObject *parent = 0 );
    virtual ~Feed();

    virtual Feed::Id id() const = 0;

    virtual QString title() const = 0;
    virtual void setTitle( const QString &title ) = 0;
    virtual QString description() const = 0;
    virtual void setDescription( const QString &description ) = 0;
    virtual QString type() const = 0;
    virtual void setType( const QString &type ) = 0;
    virtual KIcon icon() const = 0;
    virtual void setIcon( const KIcon &icon ) = 0;
    virtual QList<TagId> tags() const = 0;
    virtual void setTags( const QList<TagId> &tags ) = 0;
    virtual void addTag( const TagId &tag ) = 0;
    virtual void removeTag( const TagId &tag ) = 0;

    virtual int total() const = 0;
    virtual int unread() const = 0;

    virtual int fetchInterval() const = 0;
    virtual void setFetchInterval( int interval ) = 0;

    virtual int maxArticleNumber() const = 0;
    virtual void setMaxArticleNumber( int number ) = 0;
    virtual int maxArticleAge() const = 0;
    virtual void setMaxArticleAge( int age ) = 0;

    virtual ItemListJob* createListJob() = 0;

public Q_SLOTS:

    virtual void fetch() const = 0;
    virtual void abortFetch() const = 0;

Q_SIGNALS:
    void fetchStarted( KRss::Feed *feed );
    void fetchPercent( KRss::Feed *feed, uint percentage );
    void fetchFinished( KRss::Feed *feed );
    void fetchFailed( KRss::Feed *feed, const QString &errorMessage );
    void fetchAborted( KRss::Feed *feed );

    void changed( KRss::Feed *feed );
    void removed( KRss::Feed *feed );

    void itemsAdded( KRss::Feed *feed, const QList<KRss::Item> &items );
    void itemsChanged( KRss::Feed *feed, const QList<KRss::Item> &items );
    void itemsRemoved( KRss::Feed *feed, const QList<KRss::Item> &items );

    void totalCountChanged(  KRss::Feed *feed, int count );
    void unreadCountChanged( KRss::Feed *feed, int count );

private:
    Q_DISABLE_COPY(Feed)
};

} // namespace KRss

#endif // KRSS_FEED_H
