/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_PERSISTENTFEED_P_H
#define KRSS_PERSISTENTFEED_P_H

#include "item.h"
#include "itemlistjob.h"
#include "feedcollection.h"

#include <akonadi/itemfetchscope.h>

#include <QtCore/QHash>

namespace Akonadi {
    class ItemFetchJob;
}

namespace KRss {

class Resource;

class PersistentFeedPrivate
{
    PersistentFeed* const q;
public:

    explicit PersistentFeedPrivate( const FeedCollection &feedCollection, const Resource *resource, PersistentFeed* qq )
        : q( qq ), m_feedCollection( feedCollection ), m_resource( resource ), m_loaded( false ), m_unreadCount( 0 ), m_totalCount( 0 )
    {
    }

    void updateFromStatistics( const Akonadi::CollectionStatistics& stats );

    FeedCollection m_feedCollection;
    const Resource *m_resource;
    bool m_loaded;
    // id => item
    QHash<Item::Id, Item> m_items;
    int m_unreadCount;
    int m_totalCount;
};

class PersistentFeedItemListJobImpl : public ItemListJob {
    Q_OBJECT
public:
    explicit PersistentFeedItemListJobImpl( PersistentFeed* parent=0 ) : ItemListJob( parent ), m_feed( parent ), m_job( 0 ) {}

    /* reimp */ void setFetchScope( const Akonadi::ItemFetchScope &fetchScope ) { m_fetchScope = fetchScope; }

    /* reimp */ Akonadi::ItemFetchScope &fetchScope() { return m_fetchScope; }

    /* reimp */  QList<KRss::Item> items() const { return m_items; }

    /* reimp */ void start();

private Q_SLOTS:
    void jobDone( KJob* job );

private:
    PersistentFeed* m_feed;
    Akonadi::ItemFetchJob* m_job;
    QList<KRss::Item> m_items;
    Akonadi::ItemFetchScope m_fetchScope;
};

} // namespace KRss

#endif // KRSS_PERSISTENTFEED_P_H
