/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_PERSISTENTFEEDMODIFYJOB_H
#define KRSS_PERSISTENTFEEDMODIFYJOB_H

#include "krss_export.h"

#include <KJob>

namespace KRss {

class PersistentFeed;
class PersistentFeedModifyJobPrivate;

class KRSS_EXPORT PersistentFeedModifyJob : public KJob
{
    Q_OBJECT
public:
    enum Error {
        CouldNotModifyFeed = KJob::UserDefinedError,
        CouldNotLoadOriginalFeed,
        CouldNotRetrieveTagProvider,
        CouldNotModifyReferences
    };

public:
    explicit PersistentFeedModifyJob( const PersistentFeed *feed, QObject *parent = 0 );
    ~PersistentFeedModifyJob();

    void start();
    QString errorString() const;

private:
    friend class PersistentFeedModifyJobPrivate;
    PersistentFeedModifyJobPrivate * const d;
    Q_DISABLE_COPY( PersistentFeedModifyJob )
    Q_PRIVATE_SLOT( d, void doStart() )
    Q_PRIVATE_SLOT( d, void slotOriginalFeedLoaded( KJob *job ) )
    Q_PRIVATE_SLOT( d, void slotFeedModified( KJob *job ) )
    Q_PRIVATE_SLOT( d, void slotTagProviderRetrieved( KJob *job ) )
    Q_PRIVATE_SLOT( d, void slotReferencesModified( KJob * job ) )
};

} // namespace KRss

#endif // KRSS_PERSISTENTFEEDMODIFYJOB_H
