/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_RESOURCE_H
#define KRSS_RESOURCE_H

#include "krss_export.h"
#include "feed.h"

namespace Akonadi {
class Item;
class Collection;
class AgentInstance;
}

namespace KRss {

class ImportItemsJob;
class ImportOpmlJob;
class ResourcePrivate;

class KRSS_EXPORT Resource : public QObject
{
    Q_OBJECT

public:

    bool isValid() const;

    ImportItemsJob* createImportItemsJob( const QString& xmlUrl, const QString& sourceFile ) const;

    ImportOpmlJob* createImportOpmlJob( const KUrl& url ) const;

Q_SIGNALS:

    void feedAdded( KRss::Feed::Id id );
    void feedChanged( KRss::Feed::Id id );
    void feedRemoved( KRss::Feed::Id id );

    void statisticsChanged( KRss::Feed::Id id,
                            const Akonadi::CollectionStatistics &statistics );


    void fetchStarted( KRss::Feed::Id id );
    void fetchPercent( KRss::Feed::Id id, uint percent );
    void fetchFinished( KRss::Feed::Id id );
    void fetchFailed( KRss::Feed::Id id, const QString &errorMessage );
    void fetchAborted( KRss::Feed::Id id );

    void itemAdded( const Akonadi::Item &item, KRss::Feed::Id id );
    void itemChanged( const Akonadi::Item &item );
    void itemRemoved( const Akonadi::Item &item );

public Q_SLOTS:

    void addFeed( const QString &xmlUrl, const QString &subscriptionLabel ) const;
    void fetchFeed( KRss::Feed::Id id ) const;
    void abortFetch( KRss::Feed::Id id ) const;

private Q_SLOTS:

    void slotCollectionAdded( const Akonadi::Collection &col, const Akonadi::Collection &parent );
    void slotCollectionChanged( const Akonadi::Collection &col );
    void slotCollectionRemoved( const Akonadi::Collection &col );
    void slotCollectionStatisticsChanged( Akonadi::Collection::Id id,
                                      const Akonadi::CollectionStatistics &statistics );

    void slotFetchStarted( qlonglong id );
    void slotFetchPercent( qlonglong id, uint percent );
    void slotFetchFinished( qlonglong id );
    void slotFetchFailed( qlonglong id, const QString &errorMessage );
    void slotFetchAborted( qlonglong id );

    void slotItemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection );
    void slotItemChanged( const Akonadi::Item &item, const QSet<QByteArray> &partIdentifiers );
    void slotItemRemoved( const Akonadi::Item &item );


private:

    Q_DISABLE_COPY(Resource)

    friend class ResourceManager;
    explicit Resource( const Akonadi::AgentInstance &instance, QObject *parent = 0 );
    ~Resource();

    ResourcePrivate * const d;
};

} //namespace KRss

#endif // KRSS_RESOURCE_H
