/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "akonadi_serializer_rss.h"

#include "libkrss/item.h"
#include "libkrss/rssitem.h"
#include "libkrss/rssitemserializer.h"

#include <akonadi/item.h>

#include <QtCore/qplugin.h>

using namespace KRss;

bool SerializerPluginRss::deserialize( Akonadi::Item& item, const QByteArray& label, QIODevice& data, int version )
{
    Q_UNUSED( version )

    if ( label != Akonadi::Item::FullPayload && label != KRss::Item::HeadersPart &&
         label != KRss::Item::ContentPart )
        return false;

    RssItem rssItem;
    if ( item.hasPayload<RssItem>() ) {
        rssItem = item.payload<RssItem>();
    }

    RssItemSerializer *ser = new XmlRssItemSerializerImpl();
    if ( label == Akonadi::Item::FullPayload ) {
        ser->deserialize( rssItem, data.readAll(), RssItemSerializer::Full );
    }
    else if ( label == KRss::Item::HeadersPart ) {
        ser->deserialize( rssItem, data.readAll(), RssItemSerializer::Headers );
    }
    else if ( label == KRss::Item::ContentPart ) {
        ser->deserialize( rssItem, data.readAll(), RssItemSerializer::Content );
    }

    item.setPayload<RssItem>( rssItem );
    return true;
}

void SerializerPluginRss::serialize( const Akonadi::Item& item, const QByteArray& label, QIODevice& data, int &version )
{
    Q_UNUSED( version )

    if ( label != Akonadi::Item::FullPayload && label != KRss::Item::HeadersPart &&
         label != KRss::Item::ContentPart )
        return;

    if ( item.mimeType() != QString::fromLatin1( "application/rss+xml" ) || !item.hasPayload<RssItem>() )
        return;

    const RssItem rssItem = item.payload<RssItem>();
    RssItemSerializer *ser = new XmlRssItemSerializerImpl();
    QByteArray array;

    if ( label == Akonadi::Item::FullPayload ) {
        ser->serialize( rssItem, array, RssItemSerializer::Full );
    }
    else if ( label == KRss::Item::HeadersPart ) {
        ser->serialize( rssItem, array, RssItemSerializer::Headers );
    }
    else if ( label == KRss::Item::ContentPart ) {
        ser->serialize( rssItem, array, RssItemSerializer::Content );
    }

    data.write( array );
}

QSet<QByteArray> SerializerPluginRss::parts( const Akonadi::Item& item ) const
{
    QSet<QByteArray> partIdentifiers;

    if ( item.hasPayload<RssItem>() ) {
        const RssItem rssItem = item.payload<RssItem>();

        bool hasHeaders = false;
        bool hasContent = false;

        // if guid or hash is not empty we are sure the headers are loaded
        if ( !rssItem.guid().isEmpty() || !rssItem.link().isEmpty() ) {
            partIdentifiers << KRss::Item::HeadersPart;
            hasHeaders = true;
        }

        if ( !rssItem.description().isEmpty() || !rssItem.content().isEmpty() ) {
            partIdentifiers << KRss::Item::ContentPart;
            hasContent = true;
        }

        if ( hasHeaders && hasContent )
            partIdentifiers << Akonadi::Item::FullPayload;
    }

    return partIdentifiers;
}

Q_EXPORT_PLUGIN2( akonadi_serializer_rss, SerializerPluginRss )

#include "akonadi_serializer_rss.moc"
