/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSSRESOURCE_FETCHFEEDJOB_H
#define KRSSRESOURCE_FETCHFEEDJOB_H

#include "libkrss/feedcollection.h"

#include <syndication/syndication.h>
#include <KJob>

class FetchFeedJob : public KJob
{
    Q_OBJECT

public:

    enum Error {
        Aborted = KJob::UserDefinedError,
        Timeout,
        UnknownHost,
        FileNotFound,
        OtherRetrieverError,
        InvalidXml,
        XmlNotAccepted,
        InvalidFormat,
        ItemSyncFailed
    };

public:

    explicit FetchFeedJob( const KRss::FeedCollection &feed, QObject *parent = 0 );
    ~FetchFeedJob();

    KRss::FeedCollection feed() const;
    void abort();

    void start();
    QString errorString() const;

Q_SIGNALS:

    void progress( KRss::FeedCollection::Id id, unsigned long percentage );

private:

    friend class FetchFeedJobPrivate;
    class FetchFeedJobPrivate * const d;

    Q_DISABLE_COPY( FetchFeedJob )
    Q_PRIVATE_SLOT( d, void doStart() )
    Q_PRIVATE_SLOT( d, void slotClearNewDone( KJob* ) )
    Q_PRIVATE_SLOT( d, void slotFetchDone( Syndication::Loader *loader, Syndication::FeedPtr feed, Syndication::ErrorCode status ) )
    Q_PRIVATE_SLOT( d, void slotSyncDone( KJob *job ) )
    Q_PRIVATE_SLOT( d, void slotProgress( unsigned long percentage ) )
};

#endif // KRSSRESOURCE_FETCHFEEDJOB_H
