/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "flagsmodifyjob.h"

#include <akonadi/itemfetchjob.h>
#include <akonadi/itemmodifyjob.h>
#include <akonadi/itemfetchscope.h>

#include <KDebug>

FlagsModifyJob::FlagsModifyJob( const Akonadi::Collection &collection, QObject *parent ) :
        Job( parent ), m_collection( collection ), m_pendingJobs( 0 )
{
}

FlagsModifyJob::~FlagsModifyJob()
{
}

void FlagsModifyJob::setFlags( const Akonadi::Item::Flags &flags )
{
        m_setFlags = flags;
}

void FlagsModifyJob::clearFlags( const Akonadi::Item::Flags &flags )
{
        m_clearFlags = flags;
}

void FlagsModifyJob::doStart()
{
        Akonadi::ItemFetchJob* job = new Akonadi::ItemFetchJob( m_collection, this );
        job->fetchScope().fetchFullPayload( false );
        job->fetchScope().fetchAllAttributes( false );
        job->fetchScope().setCacheOnly( true );

        connect( job, SIGNAL( result( KJob* ) ), SLOT( slotListingDone( KJob* ) ) );
}

void FlagsModifyJob::slotListingDone( KJob *job )
{
        if ( job->error() )
                return;

        QList<Akonadi::Item> items = static_cast<Akonadi::ItemFetchJob*>( job )->items();

        Q_FOREACH( Akonadi::Item item, items ) {
                bool updated = false;
                if ( !m_setFlags.isEmpty() ) {
                        Q_FOREACH( const Akonadi::Item::Flag &flag, m_setFlags ) {
                                if ( !item.hasFlag( flag ) ) {
                                        kDebug() << "Setting" << flag << " on" << item.remoteId();
                                        item.setFlag( flag );
                                        updated = true;
                                }
                        }
                }

                if ( !m_clearFlags.isEmpty() ) {
                        Q_FOREACH( const Akonadi::Item::Flag &flag, m_clearFlags ) {
                                if ( item.hasFlag( flag ) ) {
                                        kDebug() << "Clearing" << flag << " on" << item.remoteId();
                                        item.clearFlag( flag );
                                        updated = true;
                                }
                        }
                }

                if ( updated ) {
                        m_pendingJobs++;
                        Akonadi::ItemModifyJob *job = new Akonadi::ItemModifyJob( item, this );
                        connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotModificationDone( KJob* ) ) );
                }
        }

        checkDone();
}

void FlagsModifyJob::slotModificationDone( KJob *job )
{
        if ( job->error() ) {
                kWarning() << "Failed to modify flags";
        }

        m_pendingJobs--;
        checkDone();
}

void FlagsModifyJob::checkDone()
{
        if ( m_pendingJobs > 0 )
                return;

        emitResult();
}

#include "flagsmodifyjob.moc"
